var path = require('path')
var fs = require('fs')
var config = require('../config')
var utils = require('./utils')
var webpack = require('webpack')
var merge = require('webpack-merge')
var baseWebpackConfig = require('./webpack.base.conf')
// var ExtractTextPlugin = require('extract-text-webpack-plugin')
const MiniCssExtractPlugin = require('mini-css-extract-plugin')

var modules = {}
var externals = {};
var cPath = path.join(__dirname, '../src/components')
var files = fs.readdirSync(cPath)
if (files) {
  files.slice().forEach(function (name) {
    var p = path.join(cPath, name)
    if (fs.statSync(p).isDirectory()) {
      modules[name] = p
      externals[`tg-ui/src/components/${name}`] = `tg-ui/dist/${name}`;
    }
  })
}
console.log('>>>modules', modules)

var webpackConfig = merge(baseWebpackConfig, {
  entry: modules,
  module: {
    rules: [{
      test: /\.scss$/,
      use: [
        "style-loader", // cteate style nodes from js string 
        // MiniCssExtractPlugin.loader,
        {
          loader: MiniCssExtractPlugin.loader,
          options: {
            esModule: false,
          },
        },
        "css-loader", // translates CSS into CommonJS
        "sass-loader" // compiles Sass to CSS,using Node Sass by default
      ]
    }]
  },
  devtool: config.build.productionSourceMap ? '#source-map' : false,
  output: {
    path: config.build.assetsRoot,
    filename: utils.assetsPath('[name]/index.js'),
    library: ['tg', '[name]'],
    libraryTarget: 'umd'
  },
  externals: externals,
  optimization: {
    // splitChunks: {
    //   cacheGroups: {
    //         commons: {
    //             test: /[\\/]node_modules[\\/]/,
    //             name: 'vendors',
    //             chunks: 'all'
    //         }
    //   }
    // },
  },
  plugins: [
    // 抽取 CSS 到单文件
    new MiniCssExtractPlugin({
      filename: "mycss/[name].[chunkhash:7].css",
      // chunkFilename: "[id].css"
    })
  ]
})

module.exports = webpackConfig