import createHoc from '@/utils/hoc';

/**
 *
 * @param {string} name
 * @param {VueComponent} Component
 * @param {Object} options
 * @param {boolean} options.blur 是否需要 blur 事件
 */
export default function (name, Component, options = {}) {
  return createHoc({
    name: name,
    model: true,
    inject: {
      formItem: {
        default: undefined,
      },
    },
    computed: {
      invalid () {
        if ('invalid' in this.$attrs) {
          return this.$attrs.invalid;
        }
        return this.formItem && this.formItem.state === 'error';
      },
      hasProp () {
        return this.formItem && this.formItem.prop;
      },
      loading () {
        if ('loading' in this.$attrs) {
          return this.$attrs.loading;
        }
        return this.formItem && this.formItem.state === 'validating';
      },
    },
    methods: {
      ...options.methods,
      _handleBlur (...args) {
        this.$emit('blur', ...args);
        if (!options.blur && this.hasProp) {
          this.$nextTick(() => {
            this.formItem && this.formItem.$emit('formBlur');
          });
        }
      },
      _handleChange (...args) {
        this.$emit('change', ...args);
        if (this.hasProp) {
          this.formItem && this.formItem.$emit('formChange');
        }
      },
    },
  }, {
    ...options,
    mapStateToProps (context) {
      return {
        invalid: context.invalid,
        loading: context.loading,
      };
    },
    mapMethodToListener (context) {
      return {
        blur: context._handleBlur,
        change: context._handleChange,
      };
    },
  })(Component);
};
